@@ -16,8 +16,8 @@ class Heaviside:
 
     window: float = 1.0
 
-    def __call__(self, v_mem, threshold):
-        return ((v_mem >= (threshold - self.window)).float()) / threshold
+    def __call__(self, v_mem, spike_threshold):
+        return ((v_mem >= (spike_threshold - self.window)).float()) / spike_threshold
 
 
 @dataclass
@@ -31,9 +31,9 @@ class MultiGaussian:
     sigma: float = 0.5
     grad_scale: float = 1.0
 
-    def __call__(self, v_mem, threshold):
+    def __call__(self, v_mem, spike_threshold):
         return (
-            torch.exp(-(((v_mem - threshold) - self.mu) ** 2) / (2 * self.sigma**2))
+            torch.exp(-(((v_mem - spike_threshold) - self.mu) ** 2) / (2 * self.sigma**2))
             / torch.sqrt(2 * torch.tensor(math.pi))
             / self.sigma
         ) * self.grad_scale
@@ -49,10 +49,35 @@ class SingleExponential:
     grad_width: float = 0.5
     grad_scale: float = 1.0
 
-    def __call__(self, v_mem, threshold):
-        abs_width = threshold * self.grad_width
+    def __call__(self, v_mem, spike_threshold):
+        abs_width = spike_threshold * self.grad_width
         return (
             self.grad_scale
             / abs_width
-            * torch.exp(-torch.abs(v_mem - threshold) / abs_width)
+            * torch.exp(-torch.abs(v_mem - spike_threshold) / abs_width)
         )
+
+@dataclass
+class PeriodicExponential:
+    """
+    Surrogate gradient as defined in Weidel and Sheik, 2021
+    https://arxiv.org/abs/2111.01456
+    """
+
+    grad_width: float = 0.5
+    grad_scale: float = 1.0
+
+    def __call__(self, v_mem, spike_threshold):
+        vmem_shifted = v_mem - spike_threshold / 2
+        vmem_periodic = vmem_shifted - torch.div(
+            vmem_shifted, spike_threshold, rounding_mode="floor"
+        )
+        vmem_below = vmem_shifted * (v_mem < spike_threshold)
+        vmem_above = vmem_periodic * (v_mem >= spike_threshold)
+        vmem_new = vmem_above + vmem_below
+        spikePdf = (
+            torch.exp(-torch.abs(vmem_new - spike_threshold / 2) / self.grad_width)
+            / spike_threshold
+        )
+
+        return self.grad_scale*spikePdf
